@@ -13,13 +13,14 @@ import (
 	"github.com/openshift/library-go/pkg/operator/v1helpers"
 
 	"github.com/openshift/cluster-kube-controller-manager-operator/pkg/operator/operatorclient"
+	"github.com/openshift/library-go/pkg/controller/factory"
 )
 
 // defaultRotationDay is the default rotation base for all cert rotation operations.
 const defaultRotationDay = 24 * time.Hour
 
 type CertRotationController struct {
-	certRotators []*certrotation.CertRotationController
+	certRotators []factory.Controller
 }
 
 func NewCertRotationController(
@@ -78,7 +79,7 @@ func newCertRotationController(
 		klog.Warningf("Certificate rotation base set to %q", rotationDay)
 	}
 
-	certRotator, err := certrotation.NewCertRotationController(
+	certRotator := certrotation.NewCertRotationController(
 		"CSRSigningCert",
 		certrotation.SigningRotation{
 			Namespace: operatorclient.OperatorNamespace,
@@ -115,17 +116,17 @@ func newCertRotationController(
 			EventRecorder: eventRecorder,
 		},
 		operatorClient,
+		eventRecorder,
 	)
-	if err != nil {
-		return nil, err
-	}
+
 	ret.certRotators = append(ret.certRotators, certRotator)
 
 	return ret, nil
 }
 
 func (c *CertRotationController) Run(ctx context.Context, workers int) {
+	syncCtx := context.WithValue(ctx, certrotation.RunOnceContextKey, false)
 	for _, certRotator := range c.certRotators {
-		go certRotator.Run(ctx, workers)
+		go certRotator.Run(syncCtx, workers)
 	}
 }
